import csv
import sys
from parsed_sentence import ParsedSentence
from parsed_sentence import ParsedSentenceReader

def is_noun(ps, token_id):
	pos = ps.get_postag(token_id)
	if pos.lower().startswith("n"):
		return True
	else:
		return False

if not len(sys.argv) == 2: 
	print("Usage: input_file")
	exit()

input_file_path=sys.argv[1]

input_pos="v"
input_relations=["nsubj", "dobj"]

# initialize the sentence reader
psr = ParsedSentenceReader(input_file_path)

#for each sentence
while True:
	ps = psr.read_next_sentence()
	#if there are no more sentences, break
	if not ps:
		break
		
	#get the token indexed
	indexes = ps.get_indexes();

	for id in indexes:
		# find the centern of the relation, e.g. a verb in the pattern SUBJ-V-OBJ
		pos = ps.get_postag(id)
		# if the considered word in not "of interest", continue
		if not pos.lower().startswith(input_pos):
			continue
		
		#get the lemma and children
		lemma0 = ps.get_lemma(id)	
		children = ps.get_children_indexes(id)
		
		#in the following dict, relation types are the keys, while lemmas are the values, 
		relations_lemmas_dict={}
			
		for child_id in children:
			#the elements in the pattern must be nouns
			if not is_noun(ps, child_id):
				continue
		
			# take the relation and, if the relations is not valide, continue
			relation = ps.get_relation(id, child_id)
			if not relation in input_relations:
				continue
				
			#populate the dict	
			child_lemma = ps.get_lemma(child_id)			
			relations_lemmas_dict[relation]=child_lemma
		
		# if the dict contains both a subj and an object, print the pattern
		if input_relations[0] in relations_lemmas_dict and input_relations[1] in relations_lemmas_dict:
			print(relations_lemmas_dict[input_relations[0]] + "\t-\t" + lemma0 + "\t-\t" + relations_lemmas_dict[input_relations[1]])
								