import csv
import gzip

class ParsedSentence:
	def __init__(self):
		self.ids=[]
		self.surfaces={}
		self.lemmas={}
		self.postags={}
		self.relations={}
		self.ne_categories={}

	def add_token(self, id, surface, lemma, postag, ne_category, parent_id, rel_name):
		self.ids.append(id)
		self.surfaces[id]=surface
		self.lemmas[id]=lemma
		self.postags[id]=postag
		self.ne_categories[id]=ne_category
		if parent_id not in self.relations:
			self.relations[parent_id]={}
		self.relations[parent_id][id]=rel_name

	def print_sentence(self):
		for id in self.ids:
			print("{}\t{}\t{}\t{}".format(id,self.surfaces[id], self.lemmas[id], self.postags[id]))
		print

	def get_indexes(self):
		return self.ids

	def get_father_id(self, index):
		for child_id in self.relations:
			if index in self.relations[child_id]:
				return child_id
		return None

	def get_children_indexes(self, index):
		if index in self.relations:
			return self.relations[index].keys()
		else:
			return {}

	def get_lemma(self, id):
		return self.lemmas[id]

	def get_surface(self, id):
		return self.surfaces[id]

	def get_postag(self, id):
		return self.postags[id]

	def get_ne_category(self, id):
		return self.ne_categories[id]

	def get_relation(self, parent_id, id):
		if parent_id in self.relations:
			if id in self.relations[parent_id]:
				return self.relations[parent_id][id]
		return ""

class ParsedSentenceReader:
	def __init__(self, input_file_path):
		if input_file_path.endswith("gz"):
			self.tsv = gzip.open(input_file_path, "rt")
		else:
			self.tsv = open(input_file_path)
		self.csv_reader = csv.reader(self.tsv, dialect="excel-tab")

	def read_next_sentence(self):
		ps = ParsedSentence()
		for line in self.csv_reader:
			if len(line)==7:
				id=line[0]
				surface=line[1]
				lemma=line[2]
				postag=line[3]
				ne_category=line[4]
				rel_name=line[5]
				parent_id=line[6]
				ps.add_token(id, surface, lemma, postag, ne_category, parent_id, rel_name)
			else:
				return ps

		return None
